package cc.toph.progression.item.tier;

import net.minecraft.resources.ResourceLocation;
import net.minecraft.tags.TagKey;
import net.minecraft.world.item.Item;


/**
 * The ToolMaterials interface outlines the properties and behaviors required
 * for defining custom tool materials. It provides methods for retrieving
 * information about the tool material, such as its name, repair ingredient,
 * base damage modifier, and enabled status.
 * <p>
 * Suggested implementation with Enums
 */
public interface Material<E extends Enum<E> & Material<E>> {

    /**
     * Returns the name of the enum constant in lowercase.
     *
     * @return the name of the enum constant as a lowercase string
     */
    String getName();

    /**
     * Returns RepairIngredient ResourceLocation
     *
     * @return ResourceLocation (mod:item)
     */
    ResourceLocation getIngredientLocation();

    Item getIngredientAsItem();

    /*
     * Returns RepairIngredient ResourceLocation
     *
     * @return ResourceLocation (mod:item)
     */
    TagKey<Item> getIngredientTag();

    /**
     * Checks if Material has been enabled
     *
     * @return boolean
     */
    boolean isEnabled();

    E[] getValues();
}
