package cc.toph.progression.item.tier;

import cc.toph.progression.registry.ConfigRegistry.TIER_ATTRIBUTES;
import cc.toph.progression.tag.Tags;
import net.minecraft.core.registries.BuiltInRegistries;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.tags.TagKey;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.Tier;
import net.minecraft.world.item.crafting.Ingredient;
import org.jetbrains.annotations.NotNull;

// TODO: change ingredients to tags, and correct one
public enum ToolTier implements Material<ToolTier>, Tier {
    COPPER(
            0,
            184,
            3.0F,
            1.5f,
            TIER_ATTRIBUTES.COPPER,
            22,
            Tags.ITEMS.COPPER_INGOT.tag(),
            TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    EXPOSED_COPPER(
            0,
            218,
            3.0f,
            2.0f,
            TIER_ATTRIBUTES.EXPOSED_COPPER,
            18,

            Tags.ITEMS.EXPOSED_COPPER_INGOT.tag(),
            TIER_ATTRIBUTES.EXPOSED_COPPER_ENABLED
    ),
    WEATHERED_COPPER(
            0,
            258,
            3.5f,
            2.5f,
            TIER_ATTRIBUTES.WEATHERED_COPPER,
            14,

            Tags.ITEMS.WEATHERED_COPPER_INGOT.tag(),
            TIER_ATTRIBUTES.WEATHERED_COPPER_ENABLED
    ),
    OXIDIZED_COPPER(
            0,
            304,
            4.0f,
            3.0f,
            TIER_ATTRIBUTES.OXIDIZED_COPPER,
            10,

            Tags.ITEMS.OXIDIZED_COPPER_INGOT.tag(),
            TIER_ATTRIBUTES.OXIDIZED_COPPER_ENABLED
    ),
    ROSE_GOLD(
            2,
            900,
            9.0f,
            2.0F,
            TIER_ATTRIBUTES.ROSE_GOLD,
            17,

            Tags.ITEMS.ROSE_GOLD_INGOT.tag(),
            TIER_ATTRIBUTES.ROSE_GOLD_ENABLED
    ),
    STURDY(
            3,
            1800,
            8.0f,
            3.5f,
            TIER_ATTRIBUTES.STURDY,
            14,

            Tags.ITEMS.STURDY_INGOT.tag(),
            TIER_ATTRIBUTES.STURDY_ENABLED
    ),
    STEEL(
            3,
            1200,
            7.5F,
            3.0F,
            TIER_ATTRIBUTES.STEEL,
            12,

            Tags.ITEMS.STEEL_INGOT.tag(),
            TIER_ATTRIBUTES.STEEL_ENABLED
    ),
    REFINED(
            4,
            2300,
            10.0F,
            4.5F,
            TIER_ATTRIBUTES.REFINED,
            20,

            Tags.ITEMS.REFINED_INGOT.tag(),
            TIER_ATTRIBUTES.REFINED_ENABLED
    ),
    ECHO(
            4,
            2300,
            10.0f,
            4.5F,
            TIER_ATTRIBUTES.ECHO,
            20,

            Tags.ITEMS.ECHO_INGOT.tag(),
            TIER_ATTRIBUTES.ECHO_ENABLED
    ),
    WHITHER(
            4,
            2300,
            10.0F,
            4.5F,
            TIER_ATTRIBUTES.WHITHER,
            20,

            Tags.ITEMS.WHITHER_INGOT.tag(),
            TIER_ATTRIBUTES.WHITHER_ENABLED
    ),
    VOID(
            4,
            2300,
            10.0F,
            4.5F,
            TIER_ATTRIBUTES.VOID,
            20,

            Tags.ITEMS.VOID_INGOT.tag(),
            TIER_ATTRIBUTES.VOID_ENABLED
    ),
    SHOCK(
            4,
            2300,
            10.0F,
            4.5F,
            TIER_ATTRIBUTES.SHOCK,
            20,

            Tags.ITEMS.SHOCK_INGOT.tag(),
            TIER_ATTRIBUTES.SHOCK_ENABLED
    );

    private final int level;
    private final int uses;
    private final float speed;
    private final float attackDamageBonus;
    private final float damageModifier;
    private final int enchantmentValue;
    private final TagKey<Item> repairIngredient;
    private final boolean enabled;

    ToolTier(
            int level,
            int uses,
            float speed,
            float attackDamageBonus,
            float damageModifier,
            int enchantmentValue,
            TagKey<Item> repairIngredient,
            boolean enabled
    ) {
        this.level = level;
        this.uses = uses;
        this.speed = speed;
        this.attackDamageBonus = attackDamageBonus;
        this.enchantmentValue = enchantmentValue;
        this.damageModifier = damageModifier;
        this.repairIngredient = repairIngredient;
        this.enabled = enabled;
        // TODO: Tool Effect
        // TODO: Weapon Effect
    }


    @Override
    public String getName() {
        return this.name().toLowerCase();
    }

    @Override
    public int getUses() {
        return uses;
    }

    @Override
    public float getSpeed() {
        return speed;
    }

    @Override
    public float getAttackDamageBonus() {
        return attackDamageBonus;
    }

    /**
     * Retrieves the base damage modifier associated with the tool material.
     * <p>
     * Simply Swords specific value, not related to {@link Tier#getAttackDamageBonus()}
     *
     * @return the base damage modifier as a float value
     */
    public float getDamageModifier() {
        return damageModifier;
    }

    @Override
    public ResourceLocation getIngredientLocation() {
        return repairIngredient.location();
    }

    @Override
    public TagKey<Item> getIngredientTag() {
        return repairIngredient;
    }

    @Override
    public Item getIngredientAsItem() {
        return BuiltInRegistries.ITEM.get(repairIngredient.location());
    }

    @Override
    public int getLevel() {
        return level;
    }

    @Override
    public int getEnchantmentValue() {
        return enchantmentValue;
    }

    @Override
    public boolean isEnabled() {
        return enabled;
    }

    @Override
    public @NotNull Ingredient getRepairIngredient() {
        return Ingredient.of(repairIngredient);
    }

    @Override
    public ToolTier[] getValues() {
        return values();
    }

}
