package cc.toph.progression.util;

import dev.architectury.platform.Platform;

/**
 * The CompatRecord class represents a compatibility record used for managing
 * mod integrations and extending functionality based on certain requirements.
 * This record checks whether a specific mod is present, its version is compatible,
 * and registers tool materials and weapon types if the conditions are met.
 *
 * @param modId           The unique identifier for the mod to be checked.
 * @param requiredVersion The minimum version of the mod that is required for compatibility.
 */
public record CompatRecord(
        String modId,
        String requiredVersion
) {

    /**
     * Checks whether the mod's version is loaded
     * and passes the required version check.
     */
    public boolean passCheck() {
        if (Platform.isModLoaded(modId)) {
            return Platform.getMod(modId)
                    .getVersion()
                    .compareTo(requiredVersion) >= 0;
        }
        return false;
    }

    // TODO: Should register mod items ideally
    public void register() {
        if (!passCheck()) {
            Constants.LOGGER.info("Mod " + modId + " is not loaded or is not the correct version.");
        }
        Constants.LOGGER.info("Registering compat record " + modId);
    }
}
