package cc.toph.progression.forge.datagen;

import cc.toph.progression.registry.ItemsRegistry;
import cc.toph.progression.util.Constants;
import net.minecraft.advancements.critereon.InventoryChangeTrigger;
import net.minecraft.data.PackOutput;
import net.minecraft.data.recipes.FinishedRecipe;
import net.minecraft.data.recipes.RecipeCategory;
import net.minecraft.data.recipes.RecipeProvider;
import net.minecraft.data.recipes.ShapedRecipeBuilder;
import net.minecraftforge.common.Tags;
import org.jetbrains.annotations.NotNull;

import java.util.function.Consumer;

public class ProgressionRecipes extends RecipeProvider {
    public ProgressionRecipes(PackOutput packOutput) {
        super(packOutput);
    }

    @Override
    protected void buildRecipes(@NotNull Consumer<FinishedRecipe> consumer) {
        buildArmor(consumer);
        buildSwords(consumer);
        buildTools(consumer);
        // buildSS(consumer);
    }

    private void buildArmor(@NotNull Consumer<FinishedRecipe> consumer) {
        ItemsRegistry.Armors.getArmors().forEach(armor -> {
            var tier = armor.getTier();
            switch (armor.getType()) {
                case HELMET -> ShapedRecipeBuilder.shaped(RecipeCategory.COMBAT, armor)
                        .pattern("III")
                        .pattern("I I")
                        .pattern("   ")
                        .define('I', tier.getIngredientTag())
                        .group(Constants.MOD_ID)
                        .unlockedBy("has_" + tier.getName(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                tier.getIngredientAsItem()))
                        //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                        .save(consumer);
                case CHESTPLATE -> ShapedRecipeBuilder.shaped(RecipeCategory.COMBAT, armor)
                        .pattern("I I")
                        .pattern("III")
                        .pattern("III")
                        .define('I', tier.getIngredientTag())
                        .group(Constants.MOD_ID)
                        .unlockedBy("has_" + tier.getName(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                tier.getIngredientAsItem()))
                        //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                        .save(consumer);
                case LEGGINGS -> ShapedRecipeBuilder.shaped(RecipeCategory.COMBAT, armor)
                        .pattern("III")
                        .pattern("I I")
                        .pattern("I I")
                        .define('I', tier.getIngredientTag())
                        .group(Constants.MOD_ID)
                        .unlockedBy("has_" + tier.getName(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                tier.getIngredientAsItem()))
                        //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                        .save(consumer);
                case BOOTS -> ShapedRecipeBuilder.shaped(RecipeCategory.COMBAT, armor)
                        .pattern("   ")
                        .pattern("I I")
                        .pattern("I I")
                        .define('I', tier.getIngredientTag())
                        .group(Constants.MOD_ID)
                        .unlockedBy("has_" + tier.getName(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                tier.getIngredientAsItem()))
                        //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                        .save(consumer);
                default -> throw new IllegalStateException("Unexpected value: " + armor.getType());
            }
        });
    }

    private void buildTools(@NotNull Consumer<FinishedRecipe> consumer) {
        ItemsRegistry.Tools.getAxes().forEach(axe -> {
            var tier = axe.getTier();
            ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, axe)
                    .pattern("II")
                    .pattern("SI")
                    .pattern("S ")
                    .define('I', tier.getIngredientTag())
                    .define('S', Tags.Items.RODS_WOODEN)
                    .group(Constants.MOD_ID)
                    .unlockedBy("has_" + tier.getName(), InventoryChangeTrigger.TriggerInstance.hasItems(
                            tier.getIngredientAsItem()))
                    //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                    .save(consumer);
        });
        ItemsRegistry.Tools.getPickAxes().forEach(pickAxes -> {
            var tier = pickAxes.getTier();
            ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, pickAxes)
                    .pattern("III")
                    .pattern(" S ")
                    .pattern(" S ")
                    .define('I', tier.getIngredientTag())
                    .define('S', Tags.Items.RODS_WOODEN)
                    .group(Constants.MOD_ID)
                    .unlockedBy("has_" + tier.getName(), InventoryChangeTrigger.TriggerInstance.hasItems(
                            tier.getIngredientAsItem()))
                    //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                    .save(consumer);
        });
        ItemsRegistry.Tools.getShovels().forEach(shovel -> {
            var tier = shovel.getTier();
            ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, shovel)
                    .pattern("I")
                    .pattern("S")
                    .pattern("S")
                    .define('I', tier.getIngredientTag())
                    .define('S', Tags.Items.RODS_WOODEN)
                    .group(Constants.MOD_ID)
                    .unlockedBy("has_" + tier.getName(), InventoryChangeTrigger.TriggerInstance.hasItems(
                            tier.getIngredientAsItem()))
                    //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                    .save(consumer);
        });
        ItemsRegistry.Tools.getHoes().forEach(hoe -> {
            var tier = hoe.getTier();
            ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, hoe)
                    .pattern("II")
                    .pattern("S ")
                    .pattern("S ")
                    .define('I', tier.getIngredientTag())
                    .define('S', Tags.Items.RODS_WOODEN)
                    .group(Constants.MOD_ID)
                    .unlockedBy("has_" + tier.getName(), InventoryChangeTrigger.TriggerInstance.hasItems(
                            tier.getIngredientAsItem()))
                    //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                    .save(consumer);
        });
    }

    private void buildSwords(@NotNull Consumer<FinishedRecipe> consumer) {
        ItemsRegistry.Swords.getSwords().forEach(sword -> {
            var tier = sword.getTier();
            ShapedRecipeBuilder.shaped(RecipeCategory.COMBAT, sword)
                    .pattern("I")
                    .pattern("I")
                    .pattern("S")
                    .define('I', tier.getIngredientTag())
                    .define('S', Tags.Items.RODS_WOODEN)
                    .group(Constants.MOD_ID)
                    .unlockedBy("has_" + tier.getName(), InventoryChangeTrigger.TriggerInstance.hasItems(
                            tier.getIngredientAsItem()))
                    //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                    .save(consumer);
        });
    }
}

//        if (sword instanceof SimplyCompatSwordItem<?> SCSword) {
//
/// / TODO: Some swords wont follow the usual recipe
/// / if (sword.getId() != ) {}
//
//var material = SCSword.getMaterial();
//            switch (SCSword.getType()) {
//        case CHAKRAM -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("YXY")
//                        .pattern("X X")
//                        .pattern("Y#Y")
//                        .define('Y', Items.IRON_NUGGET)
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
//        .save(consumer);
//                case CLAYMORE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern(" YX")
//                        .pattern("YXY")
//                        .pattern("#Y ")
//                        .define('Y', Items.IRON_NUGGET)
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case CUTLASS -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern(" Y")
//                        .pattern("XX")
//                        .pattern("# ")
//                        .define('Y', Items.IRON_NUGGET)
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case GLAIVE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("  X")
//                        .pattern(" #X")
//                        .pattern("#  ")
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case GREATAXE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("XXX")
//                        .pattern("Y#Y")
//                        .pattern(" # ")
//                        .define('Y', Items.IRON_NUGGET)
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case GREATHAMMER -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("XXX")
//                        .pattern("YYY")
//                        .pattern(" # ")
//                        .define('Y', Items.IRON_NUGGET)
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case HALBERD -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern(" XY")
//                        .pattern("X#X")
//                        .pattern("#  ")
//                        .define('Y', Items.IRON_NUGGET)
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case KATANA -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("   ")
//                        .pattern("#XX")
//                        .pattern("   ")
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case LONGSWORD -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("#  ")
//                        .pattern(" X ")
//                        .pattern("  X")
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case RAPIER -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("  X")
//                        .pattern(" X ")
//                        .pattern("#  ")
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case SAI -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("  ")
//                        .pattern(" X")
//                        .pattern("# ")
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case SCYTHE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("XX#")
//                        .pattern("X# ")
//                        .pattern("#  ")
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case SPEAR -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("  X")
//                        .pattern(" # ")
//                        .pattern("#  ")
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case TWINBLADE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern("  X")
//                        .pattern(" # ")
//                        .pattern("X  ")
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//                case WARGLAIVE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
//                        .pattern(" Y ")
//                        .pattern("X#X")
//                        .pattern("   ")
//                        .define('Y', Items.IRON_NUGGET)
//                        .define('X', SCSword.getTier().getRepairIngredient())
//        .define('#', Tags.Items.RODS_WOODEN)
//                        .group(Constants.MOD_ID)
//                        .unlockedBy("has_" + tier.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
//        BuiltInRegistries.ITEM.get(tier.getIngredientLocation())))
//        .save(consumer);
//default -> throw new IllegalStateException("Unexpected value: " + SCSword.getType());
//        }
//        } else {
//        throw new IllegalStateException("Unexpected item type: " + sword.getClass().getName() + "or");
//        }